@@ -39,9 +39,12 @@
 #include "i965_encoder.h"
 
 static void
-gen6_mfc_pipe_mode_select(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+gen6_mfc_pipe_mode_select(VADriverContextP ctx,
+                          struct gen6_encoder_context *gen6_encoder_context,
+                          struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
 
     BEGIN_BCS_BATCH(batch, 4);
 
@@ -73,9 +76,11 @@ gen6_mfc_pipe_mode_select(VADriverContextP ctx, struct gen6_encoder_context *gen
 static void
 gen7_mfc_pipe_mode_select(VADriverContextP ctx,
                           int standard_select,
-                          struct gen6_encoder_context *gen6_encoder_context)
+                          struct gen6_encoder_context *gen6_encoder_context,
+                          struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
 
     assert(standard_select == MFX_FORMAT_MPEG2 ||
            standard_select == MFX_FORMAT_AVC);
@@ -107,11 +112,15 @@ gen7_mfc_pipe_mode_select(VADriverContextP ctx,
 }
 
 static void
-gen6_mfc_surface_state(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+gen6_mfc_surface_state(VADriverContextP ctx,
+                       struct gen6_encoder_context *gen6_encoder_context,
+                       struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     struct gen6_mfc_context *mfc_context = &gen6_encoder_context->mfc_context;
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     BEGIN_BCS_BATCH(batch, 6);
 
     OUT_BCS_BATCH(batch, MFX_SURFACE_STATE | (6 - 2));
@@ -135,11 +144,15 @@ gen6_mfc_surface_state(VADriverContextP ctx, struct gen6_encoder_context *gen6_e
 }
 
 static void
-gen7_mfc_surface_state(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+gen7_mfc_surface_state(VADriverContextP ctx,
+                       struct gen6_encoder_context *gen6_encoder_context,
+                       struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     struct gen6_mfc_context *mfc_context = &gen6_encoder_context->mfc_context;
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     BEGIN_BCS_BATCH(batch, 6);
 
     OUT_BCS_BATCH(batch, MFX_SURFACE_STATE | (6 - 2));
@@ -163,12 +176,16 @@ gen7_mfc_surface_state(VADriverContextP ctx, struct gen6_encoder_context *gen6_e
 }
 
 static void
-gen6_mfc_pipe_buf_addr_state(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+gen6_mfc_pipe_buf_addr_state(VADriverContextP ctx,
+                             struct gen6_encoder_context *gen6_encoder_context,
+                             struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     struct gen6_mfc_context *mfc_context = &gen6_encoder_context->mfc_context;
     int i;
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     BEGIN_BCS_BATCH(batch, 24);
 
     OUT_BCS_BATCH(batch, MFX_PIPE_BUF_ADDR_STATE | (24 - 2));
@@ -206,11 +223,15 @@ gen6_mfc_pipe_buf_addr_state(VADriverContextP ctx, struct gen6_encoder_context *
 }
 
 static void
-gen6_mfc_ind_obj_base_addr_state(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+gen6_mfc_ind_obj_base_addr_state(VADriverContextP ctx,
+                                 struct gen6_encoder_context *gen6_encoder_context,
+                                 struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     struct gen6_vme_context *vme_context = &gen6_encoder_context->vme_context;
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     BEGIN_BCS_BATCH(batch, 11);
 
     OUT_BCS_BATCH(batch, MFX_IND_OBJ_BASE_ADDR_STATE | (11 - 2));
@@ -231,11 +252,15 @@ gen6_mfc_ind_obj_base_addr_state(VADriverContextP ctx, struct gen6_encoder_conte
 }
 
 static void
-gen7_mfc_ind_obj_base_addr_state(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+gen7_mfc_ind_obj_base_addr_state(VADriverContextP ctx,
+                                 struct gen6_encoder_context *gen6_encoder_context,
+                                 struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     struct gen6_vme_context *vme_context = &gen6_encoder_context->vme_context;
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     BEGIN_BCS_BATCH(batch, 11);
 
     OUT_BCS_BATCH(batch, MFX_IND_OBJ_BASE_ADDR_STATE | (11 - 2));
@@ -256,11 +281,15 @@ gen7_mfc_ind_obj_base_addr_state(VADriverContextP ctx, struct gen6_encoder_conte
 }
 
 static void
-gen6_mfc_bsp_buf_base_addr_state(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+gen6_mfc_bsp_buf_base_addr_state(VADriverContextP ctx,
+                                 struct gen6_encoder_context *gen6_encoder_context,
+                                 struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     struct gen6_mfc_context *mfc_context = &gen6_encoder_context->mfc_context;
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     BEGIN_BCS_BATCH(batch, 4);
 
     OUT_BCS_BATCH(batch, MFX_BSP_BUF_BASE_ADDR_STATE | (4 - 2));
@@ -274,14 +303,17 @@ gen6_mfc_bsp_buf_base_addr_state(VADriverContextP ctx, struct gen6_encoder_conte
 }
 
 static void
-gen6_mfc_avc_img_state(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+gen6_mfc_avc_img_state(VADriverContextP ctx,
+                       struct gen6_encoder_context *gen6_encoder_context,
+                       struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     struct gen6_mfc_context *mfc_context = &gen6_encoder_context->mfc_context;
-
     int width_in_mbs = (mfc_context->surface_state.width + 15) / 16;
     int height_in_mbs = (mfc_context->surface_state.height + 15) / 16;
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     BEGIN_BCS_BATCH(batch, 13);
     OUT_BCS_BATCH(batch, MFX_AVC_IMG_STATE | (13 - 2));
     OUT_BCS_BATCH(batch, 
@@ -328,14 +360,17 @@ gen6_mfc_avc_img_state(VADriverContextP ctx, struct gen6_encoder_context *gen6_e
 }
 
 static void
-gen7_mfc_avc_img_state(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+gen7_mfc_avc_img_state(VADriverContextP ctx,
+                       struct gen6_encoder_context *gen6_encoder_context,
+                       struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     struct gen6_mfc_context *mfc_context = &gen6_encoder_context->mfc_context;
-
     int width_in_mbs = (mfc_context->surface_state.width + 15) / 16;
     int height_in_mbs = (mfc_context->surface_state.height + 15) / 16;
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     BEGIN_BCS_BATCH(batch, 16);
     OUT_BCS_BATCH(batch, MFX_AVC_IMG_STATE | (16 - 2));
     OUT_BCS_BATCH(batch,
@@ -385,49 +420,16 @@ gen7_mfc_avc_img_state(VADriverContextP ctx, struct gen6_encoder_context *gen6_e
     ADVANCE_BCS_BATCH(batch);
 }
 
-static void gen6_mfc_avc_directmode_state(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
-{
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
-    int i;
-
-    BEGIN_BCS_BATCH(batch, 69);
-
-    OUT_BCS_BATCH(batch, MFX_AVC_DIRECTMODE_STATE | (69 - 2));
-    //TODO: reference DMV
-    for(i = 0; i < 16; i++){
-        OUT_BCS_BATCH(batch, 0);
-        OUT_BCS_BATCH(batch, 0);
-    }
-
-    //TODO: current DMV just for test
-#if 0
-    OUT_BCS_RELOC(batch, mfc_context->direct_mv_buffers[0].bo,
-                  I915_GEM_DOMAIN_INSTRUCTION, I915_GEM_DOMAIN_INSTRUCTION,
-                  0);
-#else
-    //drm_intel_bo_pin(mfc_context->direct_mv_buffers[0].bo, 0x1000);
-    //OUT_BCS_BATCH(batch, mfc_context->direct_mv_buffers[0].bo->offset);
-    OUT_BCS_BATCH(batch, 0);
-#endif
-
-
-    OUT_BCS_BATCH(batch, 0);
-
-    //TODO: POL list
-    for(i = 0; i < 34; i++) {
-        OUT_BCS_BATCH(batch, 0);
-    }
-
-    ADVANCE_BCS_BATCH(batch);
-}
-
 static void gen6_mfc_avc_slice_state(VADriverContextP ctx,
                                      int intra_slice,
-                                     struct gen6_encoder_context *gen6_encoder_context)
+                                     struct gen6_encoder_context *gen6_encoder_context,
+                                     struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     struct gen6_mfc_context *mfc_context = &gen6_encoder_context->mfc_context;
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     BEGIN_BCS_BATCH(batch, 11);;
 
     OUT_BCS_BATCH(batch, MFX_AVC_SLICE_STATE | (11 - 2) );
@@ -470,11 +472,15 @@ static void gen6_mfc_avc_slice_state(VADriverContextP ctx,
 
     ADVANCE_BCS_BATCH(batch);
 }
-static void gen6_mfc_avc_qm_state(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+static void gen6_mfc_avc_qm_state(VADriverContextP ctx,
+                                  struct gen6_encoder_context *gen6_encoder_context,
+                                  struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     int i;
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     BEGIN_BCS_BATCH(batch, 58);
 
     OUT_BCS_BATCH(batch, MFX_AVC_QM_STATE | 56);
@@ -486,11 +492,15 @@ static void gen6_mfc_avc_qm_state(VADriverContextP ctx, struct gen6_encoder_cont
     ADVANCE_BCS_BATCH(batch);
 }
 
-static void gen6_mfc_avc_fqm_state(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+static void gen6_mfc_avc_fqm_state(VADriverContextP ctx,
+                                   struct gen6_encoder_context *gen6_encoder_context,
+                                   struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     int i;
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     BEGIN_BCS_BATCH(batch, 113);
     OUT_BCS_BATCH(batch, MFC_AVC_FQM_STATE | (113 - 2));
 
@@ -506,11 +516,14 @@ gen7_mfc_qm_state(VADriverContextP ctx,
                   int qm_type,
                   unsigned int *qm,
                   int qm_length,
-                  struct gen6_encoder_context *gen6_encoder_context)
+                  struct gen6_encoder_context *gen6_encoder_context,
+                  struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     unsigned int qm_buffer[16];
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     assert(qm_length <= 16);
     assert(sizeof(*qm) == 4);
     memcpy(qm_buffer, qm, qm_length * 4);
@@ -522,7 +535,9 @@ gen7_mfc_qm_state(VADriverContextP ctx,
     ADVANCE_BCS_BATCH(batch);
 }
 
-static void gen7_mfc_avc_qm_state(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+static void gen7_mfc_avc_qm_state(VADriverContextP ctx,
+                                  struct gen6_encoder_context *gen6_encoder_context,
+                                  struct intel_batchbuffer *batch)
 {
     unsigned int qm[16] = {
         0x10101010, 0x10101010, 0x10101010, 0x10101010,
@@ -531,10 +546,10 @@ static void gen7_mfc_avc_qm_state(VADriverContextP ctx, struct gen6_encoder_cont
         0x10101010, 0x10101010, 0x10101010, 0x10101010
     };
 
-    gen7_mfc_qm_state(ctx, MFX_QM_AVC_4X4_INTRA_MATRIX, qm, 12, gen6_encoder_context);
-    gen7_mfc_qm_state(ctx, MFX_QM_AVC_4X4_INTER_MATRIX, qm, 12, gen6_encoder_context);
-    gen7_mfc_qm_state(ctx, MFX_QM_AVC_8x8_INTRA_MATRIX, qm, 16, gen6_encoder_context);
-    gen7_mfc_qm_state(ctx, MFX_QM_AVC_8x8_INTER_MATRIX, qm, 16, gen6_encoder_context);
+    gen7_mfc_qm_state(ctx, MFX_QM_AVC_4X4_INTRA_MATRIX, qm, 12, gen6_encoder_context, batch);
+    gen7_mfc_qm_state(ctx, MFX_QM_AVC_4X4_INTER_MATRIX, qm, 12, gen6_encoder_context, batch);
+    gen7_mfc_qm_state(ctx, MFX_QM_AVC_8x8_INTRA_MATRIX, qm, 16, gen6_encoder_context, batch);
+    gen7_mfc_qm_state(ctx, MFX_QM_AVC_8x8_INTER_MATRIX, qm, 16, gen6_encoder_context, batch);
 }
 
 static void
@@ -542,11 +557,14 @@ gen7_mfc_fqm_state(VADriverContextP ctx,
                    int fqm_type,
                    unsigned int *fqm,
                    int fqm_length,
-                   struct gen6_encoder_context *gen6_encoder_context)
+                   struct gen6_encoder_context *gen6_encoder_context,
+                   struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     unsigned int fqm_buffer[32];
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     assert(fqm_length <= 32);
     assert(sizeof(*fqm) == 4);
     memcpy(fqm_buffer, fqm, fqm_length * 4);
@@ -558,7 +576,9 @@ gen7_mfc_fqm_state(VADriverContextP ctx,
     ADVANCE_BCS_BATCH(batch);
 }
 
-static void gen7_mfc_avc_fqm_state(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+static void gen7_mfc_avc_fqm_state(VADriverContextP ctx,
+                                   struct gen6_encoder_context *gen6_encoder_context,
+                                   struct intel_batchbuffer *batch)
 {
     unsigned int qm[32] = {
         0x10001000, 0x10001000, 0x10001000, 0x10001000,
@@ -571,17 +591,21 @@ static void gen7_mfc_avc_fqm_state(VADriverContextP ctx, struct gen6_encoder_con
         0x10001000, 0x10001000, 0x10001000, 0x10001000
     };
 
-    gen7_mfc_fqm_state(ctx, MFX_QM_AVC_4X4_INTRA_MATRIX, qm, 24, gen6_encoder_context);
-    gen7_mfc_fqm_state(ctx, MFX_QM_AVC_4X4_INTER_MATRIX, qm, 24, gen6_encoder_context);
-    gen7_mfc_fqm_state(ctx, MFX_QM_AVC_8x8_INTRA_MATRIX, qm, 32, gen6_encoder_context);
-    gen7_mfc_fqm_state(ctx, MFX_QM_AVC_8x8_INTER_MATRIX, qm, 32, gen6_encoder_context);
+    gen7_mfc_fqm_state(ctx, MFX_QM_AVC_4X4_INTRA_MATRIX, qm, 24, gen6_encoder_context, batch);
+    gen7_mfc_fqm_state(ctx, MFX_QM_AVC_4X4_INTER_MATRIX, qm, 24, gen6_encoder_context, batch);
+    gen7_mfc_fqm_state(ctx, MFX_QM_AVC_8x8_INTRA_MATRIX, qm, 32, gen6_encoder_context, batch);
+    gen7_mfc_fqm_state(ctx, MFX_QM_AVC_8x8_INTER_MATRIX, qm, 32, gen6_encoder_context, batch);
 }
 
-static void gen6_mfc_avc_ref_idx_state(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+static void gen6_mfc_avc_ref_idx_state(VADriverContextP ctx,
+                                       struct gen6_encoder_context *gen6_encoder_context,
+                                       struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     int i;
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     BEGIN_BCS_BATCH(batch, 10);
 
     OUT_BCS_BATCH(batch, MFX_AVC_REF_IDX_STATE | 8);
@@ -595,32 +619,16 @@ static void gen6_mfc_avc_ref_idx_state(VADriverContextP ctx, struct gen6_encoder
     ADVANCE_BCS_BATCH(batch);
 }
 	
-static void
-gen6_mfc_avc_insert_object(VADriverContextP ctx, int flush_data, struct gen6_encoder_context *gen6_encoder_context)
-{
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
-
-    BEGIN_BCS_BATCH(batch, 4);
-
-    OUT_BCS_BATCH(batch, MFC_AVC_INSERT_OBJECT | (4 -2 ) );
-    OUT_BCS_BATCH(batch, (32<<8) | 
-                  (1 << 3) |
-                  (1 << 2) |
-                  (flush_data << 1) |
-                  (1<<0) );
-    OUT_BCS_BATCH(batch, 0x00000003);
-    OUT_BCS_BATCH(batch, 0xABCD1234);
-
-    ADVANCE_BCS_BATCH(batch);
-}
-
 static int
 gen6_mfc_avc_pak_object_intra(VADriverContextP ctx, int x, int y, int end_mb, int qp,unsigned int *msg,
-                              struct gen6_encoder_context *gen6_encoder_context)
+                              struct gen6_encoder_context *gen6_encoder_context,
+                              struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     int len_in_dwords = 11;
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     BEGIN_BCS_BATCH(batch, len_in_dwords);
 
     OUT_BCS_BATCH(batch, MFC_AVC_PAK_OBJECT | (len_in_dwords - 2));
@@ -651,11 +659,13 @@ gen6_mfc_avc_pak_object_intra(VADriverContextP ctx, int x, int y, int end_mb, in
 }
 
 static int gen6_mfc_avc_pak_object_inter(VADriverContextP ctx, int x, int y, int end_mb, int qp, unsigned int offset,
-                                         struct gen6_encoder_context *gen6_encoder_context)
+                                         struct gen6_encoder_context *gen6_encoder_context, struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     int len_in_dwords = 11;
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     BEGIN_BCS_BATCH(batch, len_in_dwords);
 
     OUT_BCS_BATCH(batch, MFC_AVC_PAK_OBJECT | (len_in_dwords - 2));
@@ -696,12 +706,16 @@ static int gen6_mfc_avc_pak_object_inter(VADriverContextP ctx, int x, int y, int
     return len_in_dwords;
 }
 
-static void gen6_mfc_init(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+static void gen6_mfc_init(VADriverContextP ctx,
+                          struct encode_state *encode_state,
+                          struct gen6_encoder_context *gen6_encoder_context)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
     struct gen6_mfc_context *mfc_context = &gen6_encoder_context->mfc_context;
     dri_bo *bo;
     int i;
+    VAEncSequenceParameterBufferH264 *pSequenceParameter = (VAEncSequenceParameterBufferH264 *)encode_state->seq_param->buffer;
+    int width_in_mbs = pSequenceParameter->picture_width_in_mbs;
 
     /*Encode common setup for MFC*/
     dri_bo_unreference(mfc_context->post_deblocking_output.bo);
@@ -730,7 +744,7 @@ static void gen6_mfc_init(VADriverContextP ctx, struct gen6_encoder_context *gen
     dri_bo_unreference(mfc_context->intra_row_store_scratch_buffer.bo);
     bo = dri_bo_alloc(i965->intel.bufmgr,
                       "Buffer",
-                      128 * 64,
+                      width_in_mbs * 64,
                       64);
     assert(bo);
     mfc_context->intra_row_store_scratch_buffer.bo = bo;
@@ -738,7 +752,7 @@ static void gen6_mfc_init(VADriverContextP ctx, struct gen6_encoder_context *gen
     dri_bo_unreference(mfc_context->deblocking_filter_row_store_scratch_buffer.bo);
     bo = dri_bo_alloc(i965->intel.bufmgr,
                       "Buffer",
-                      49152,  /* 6 * 128 * 64 */
+                      4 * width_in_mbs * 64,  /* 4 * width_in_mbs * 64 */
                       64);
     assert(bo);
     mfc_context->deblocking_filter_row_store_scratch_buffer.bo = bo;
@@ -746,7 +760,7 @@ static void gen6_mfc_init(VADriverContextP ctx, struct gen6_encoder_context *gen
     dri_bo_unreference(mfc_context->bsd_mpc_row_store_scratch_buffer.bo);
     bo = dri_bo_alloc(i965->intel.bufmgr,
                       "Buffer",
-                      12288, /* 1.5 * 128 * 64 */
+                      128 * width_in_mbs, /* 2 * widht_in_mbs * 64 */
                       0x1000);
     assert(bo);
     mfc_context->bsd_mpc_row_store_scratch_buffer.bo = bo;
@@ -757,7 +771,7 @@ void gen6_mfc_avc_pipeline_programing(VADriverContextP ctx,
                                       struct gen6_encoder_context *gen6_encoder_context)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
+    struct intel_batchbuffer *main_batch = gen6_encoder_context->base.batch;
     struct gen6_mfc_context *mfc_context = &gen6_encoder_context->mfc_context;
     struct gen6_vme_context *vme_context = &gen6_encoder_context->vme_context;
     VAEncSequenceParameterBufferH264 *pSequenceParameter = (VAEncSequenceParameterBufferH264 *)encode_state->seq_param->buffer;
@@ -768,8 +782,9 @@ void gen6_mfc_avc_pipeline_programing(VADriverContextP ctx,
     int width_in_mbs = (mfc_context->surface_state.width + 15) / 16;
     int height_in_mbs = (mfc_context->surface_state.height + 15) / 16;
     int x,y;
+    struct intel_batchbuffer *batch = intel_batchbuffer_new(&i965->intel, I915_EXEC_BSD, width_in_mbs * height_in_mbs * 12 * 4 + 0x800);
 
-    intel_batchbuffer_start_atomic_bcs(batch, 0x1000); 
+    intel_batchbuffer_start_atomic_bcs(batch, width_in_mbs * height_in_mbs * 12 * 4 + 0x700);
 
     if (is_intra) {
         dri_bo_map(vme_context->vme_output.bo , 1);
@@ -785,39 +800,39 @@ void gen6_mfc_avc_pipeline_programing(VADriverContextP ctx,
                 intel_batchbuffer_emit_mi_flush(batch);
                 
                 if (IS_GEN7(i965->intel.device_id)) {
-                    gen7_mfc_pipe_mode_select(ctx, MFX_FORMAT_AVC, gen6_encoder_context);
-                    gen7_mfc_surface_state(ctx, gen6_encoder_context);
-                    gen7_mfc_ind_obj_base_addr_state(ctx, gen6_encoder_context);
+                    gen7_mfc_pipe_mode_select(ctx, MFX_FORMAT_AVC, gen6_encoder_context, batch);
+                    gen7_mfc_surface_state(ctx, gen6_encoder_context, batch);
+                    gen7_mfc_ind_obj_base_addr_state(ctx, gen6_encoder_context, batch);
                 } else {
-                    gen6_mfc_pipe_mode_select(ctx, gen6_encoder_context);
-                    gen6_mfc_surface_state(ctx, gen6_encoder_context);
-                    gen6_mfc_ind_obj_base_addr_state(ctx, gen6_encoder_context);
+                    gen6_mfc_pipe_mode_select(ctx, gen6_encoder_context, batch);
+                    gen6_mfc_surface_state(ctx, gen6_encoder_context, batch);
+                    gen6_mfc_ind_obj_base_addr_state(ctx, gen6_encoder_context, batch);
                 }
 
-                gen6_mfc_pipe_buf_addr_state(ctx, gen6_encoder_context);
-                gen6_mfc_bsp_buf_base_addr_state(ctx, gen6_encoder_context);
+                gen6_mfc_pipe_buf_addr_state(ctx, gen6_encoder_context, batch);
+                gen6_mfc_bsp_buf_base_addr_state(ctx, gen6_encoder_context, batch);
 
                 if (IS_GEN7(i965->intel.device_id)) {
-                    gen7_mfc_avc_img_state(ctx, gen6_encoder_context);
-                    gen7_mfc_avc_qm_state(ctx, gen6_encoder_context);
-                    gen7_mfc_avc_fqm_state(ctx, gen6_encoder_context);
+                    gen7_mfc_avc_img_state(ctx, gen6_encoder_context, batch);
+                    gen7_mfc_avc_qm_state(ctx, gen6_encoder_context, batch);
+                    gen7_mfc_avc_fqm_state(ctx, gen6_encoder_context, batch);
                 } else {
-                    gen6_mfc_avc_img_state(ctx, gen6_encoder_context);
-                    gen6_mfc_avc_qm_state(ctx, gen6_encoder_context);
-                    gen6_mfc_avc_fqm_state(ctx, gen6_encoder_context);
+                    gen6_mfc_avc_img_state(ctx, gen6_encoder_context, batch);
+                    gen6_mfc_avc_qm_state(ctx, gen6_encoder_context, batch);
+                    gen6_mfc_avc_fqm_state(ctx, gen6_encoder_context, batch);
                 }
 
-                gen6_mfc_avc_ref_idx_state(ctx, gen6_encoder_context);
-                gen6_mfc_avc_slice_state(ctx, is_intra, gen6_encoder_context);
+                gen6_mfc_avc_ref_idx_state(ctx, gen6_encoder_context, batch);
+                gen6_mfc_avc_slice_state(ctx, is_intra, gen6_encoder_context, batch);
                 emit_new_state = 0;
             }
 
             if (is_intra) {
                 assert(msg);
-                object_len_in_bytes = gen6_mfc_avc_pak_object_intra(ctx, x, y, last_mb, qp, msg, gen6_encoder_context);
+                object_len_in_bytes = gen6_mfc_avc_pak_object_intra(ctx, x, y, last_mb, qp, msg, gen6_encoder_context, batch);
                 msg += 4;
             } else {
-                object_len_in_bytes = gen6_mfc_avc_pak_object_inter(ctx, x, y, last_mb, qp, offset, gen6_encoder_context);
+                object_len_in_bytes = gen6_mfc_avc_pak_object_inter(ctx, x, y, last_mb, qp, offset, gen6_encoder_context, batch);
                 offset += 64;
             }
 
@@ -832,8 +847,30 @@ void gen6_mfc_avc_pipeline_programing(VADriverContextP ctx,
 
     if (is_intra)
         dri_bo_unmap(vme_context->vme_output.bo);
-	
+
+    intel_batchbuffer_align(batch, 8);
+
+    BEGIN_BCS_BATCH(batch, 2);
+    OUT_BCS_BATCH(batch, 0);
+    OUT_BCS_BATCH(batch, MI_BATCH_BUFFER_END);
+    ADVANCE_BCS_BATCH(batch);
+
     intel_batchbuffer_end_atomic(batch);
+
+    /* chain to the main batch buffer */
+    intel_batchbuffer_start_atomic_bcs(main_batch, 0x100);
+    intel_batchbuffer_emit_mi_flush(main_batch);
+    BEGIN_BCS_BATCH(main_batch, 2);
+    OUT_BCS_BATCH(main_batch, MI_BATCH_BUFFER_START | (1 << 8));
+    OUT_BCS_RELOC(main_batch,
+                  batch->buffer,
+                  I915_GEM_DOMAIN_COMMAND, 0,
+                  0);
+    ADVANCE_BCS_BATCH(main_batch);
+    intel_batchbuffer_end_atomic(main_batch);
+
+    // end programing             
+    intel_batchbuffer_free(batch);	
 }
 
 static VAStatus gen6_mfc_avc_prepare(VADriverContextP ctx, 
@@ -921,7 +958,7 @@ gen6_mfc_avc_encode_picture(VADriverContextP ctx,
                             struct encode_state *encode_state,
                             struct gen6_encoder_context *gen6_encoder_context)
 {
-    gen6_mfc_init(ctx, gen6_encoder_context);
+    gen6_mfc_init(ctx, encode_state, gen6_encoder_context);
     gen6_mfc_avc_prepare(ctx, encode_state, gen6_encoder_context);
     gen6_mfc_run(ctx, encode_state, gen6_encoder_context);
     gen6_mfc_stop(ctx, encode_state, gen6_encoder_context);
