@@ -26,22 +26,19 @@
  *
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <assert.h>
+#ifndef HAVE_GEN_AVC_SURFACE
+#define HAVE_GEN_AVC_SURFACE 1
+#endif
 
+#include "sysdeps.h"
 #include "intel_batchbuffer.h"
 #include "intel_driver.h"
-
 #include "i965_defines.h"
 #include "i965_drv_video.h"
 #include "i965_decoder_utils.h"
 
 #include "gen6_mfd.h"
 
-#define DMV_SIZE        0x88000 /* 557056 bytes for a frame */
-
 static const uint32_t zigzag_direct[64] = {
     0,   1,  8, 16,  9,  2,  3, 10,
     17, 24, 32, 25, 18, 11,  4,  5,
@@ -169,35 +166,20 @@ gen6_mfd_avc_frame_store_index(VADriverContextP ctx,
     }
 }
 
-static void 
-gen6_mfd_free_avc_surface(void **data)
-{
-    struct gen6_avc_surface *gen6_avc_surface = *data;
-
-    if (!gen6_avc_surface)
-        return;
-
-    dri_bo_unreference(gen6_avc_surface->dmv_top);
-    gen6_avc_surface->dmv_top = NULL;
-    dri_bo_unreference(gen6_avc_surface->dmv_bottom);
-    gen6_avc_surface->dmv_bottom = NULL;
-
-    free(gen6_avc_surface);
-    *data = NULL;
-}
-
 static void
 gen6_mfd_init_avc_surface(VADriverContextP ctx, 
                           VAPictureParameterBufferH264 *pic_param,
                           struct object_surface *obj_surface)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct gen6_avc_surface *gen6_avc_surface = obj_surface->private_data;
+    GenAvcSurface *gen6_avc_surface = obj_surface->private_data;
+    int height_in_mbs;
 
-    obj_surface->free_private_data = gen6_mfd_free_avc_surface;
+    obj_surface->free_private_data = gen_free_avc_surface;
+    height_in_mbs = ((pic_param->picture_height_in_mbs_minus1 + 1) & 0xff); /* frame height */
 
     if (!gen6_avc_surface) {
-        gen6_avc_surface = calloc(sizeof(struct gen6_avc_surface), 1);
+        gen6_avc_surface = calloc(sizeof(GenAvcSurface), 1);
         assert((obj_surface->size & 0x3f) == 0);
         obj_surface->private_data = gen6_avc_surface;
     }
@@ -208,7 +190,7 @@ gen6_mfd_init_avc_surface(VADriverContextP ctx,
     if (gen6_avc_surface->dmv_top == NULL) {
         gen6_avc_surface->dmv_top = dri_bo_alloc(i965->intel.bufmgr,
                                                  "direct mv w/r buffer",
-                                                 DMV_SIZE,
+                                                 128 * height_in_mbs * 64,      /* scalable with frame height */
                                                  0x1000);
     }
 
@@ -216,7 +198,7 @@ gen6_mfd_init_avc_surface(VADriverContextP ctx,
         gen6_avc_surface->dmv_bottom == NULL) {
         gen6_avc_surface->dmv_bottom = dri_bo_alloc(i965->intel.bufmgr,
                                                     "direct mv w/r buffer",
-                                                    DMV_SIZE,
+                                                    128 * height_in_mbs * 64,   /* scalable with frame height */
                                                     0x1000);
     }
 }
@@ -412,27 +394,6 @@ gen6_mfd_bsp_buf_base_addr_state(VADriverContextP ctx,
     ADVANCE_BCS_BATCH(batch);
 }
 
-static void
-gen6_mfd_aes_state(VADriverContextP ctx,
-                   struct decode_state *decode_state,
-                   int standard_select)
-{
-    /* FIXME */
-}
-
-static void
-gen6_mfd_wait(VADriverContextP ctx,
-              struct decode_state *decode_state,
-              int standard_select,
-              struct gen6_mfd_context *gen6_mfd_context)
-{
-    struct intel_batchbuffer *batch = gen6_mfd_context->base.batch;
-
-    BEGIN_BCS_BATCH(batch, 1);
-    OUT_BCS_BATCH(batch, MFX_WAIT | (1 << 8));
-    ADVANCE_BCS_BATCH(batch);
-}
-
 static void
 gen6_mfd_avc_img_state(VADriverContextP ctx,
                        struct decode_state *decode_state,
@@ -575,7 +536,7 @@ gen6_mfd_avc_directmode_state(VADriverContextP ctx,
     struct i965_driver_data *i965 = i965_driver_data(ctx);
     struct intel_batchbuffer *batch = gen6_mfd_context->base.batch;
     struct object_surface *obj_surface;
-    struct gen6_avc_surface *gen6_avc_surface;
+    GenAvcSurface *gen6_avc_surface;
     VAPictureH264 *va_pic;
     int i, j;
 
@@ -885,6 +846,7 @@ gen6_mfd_avc_bsd_object(VADriverContextP ctx,
                   (0 << 8));
     OUT_BCS_BATCH(batch,
                   ((slice_data_bit_offset >> 3) << 16) |
+                  (1 << 7)  |
                   (1 << 6)  |
                   ((0x7 - (slice_data_bit_offset & 0x7)) << 0));
     OUT_BCS_BATCH(batch, 0);
@@ -929,6 +891,7 @@ gen6_mfd_avc_decode_init(VADriverContextP ctx,
     struct object_surface *obj_surface;
     dri_bo *bo;
     int i, j, enable_avc_ildb = 0;
+    int width_in_mbs;
 
     for (j = 0; j < decode_state->num_slice_params && enable_avc_ildb == 0; j++) {
         assert(decode_state->slice_params && decode_state->slice_params[j]->buffer);
@@ -954,6 +917,7 @@ gen6_mfd_avc_decode_init(VADriverContextP ctx,
     assert(decode_state->pic_param && decode_state->pic_param->buffer);
     pic_param = (VAPictureParameterBufferH264 *)decode_state->pic_param->buffer;
     gen6_mfd_avc_frame_store_index(ctx, pic_param, gen6_mfd_context);
+    width_in_mbs = ((pic_param->picture_width_in_mbs_minus1 + 1) & 0xff);
 
     /* Current decoded picture */
     va_pic = &pic_param->CurrPic;
@@ -963,6 +927,17 @@ gen6_mfd_avc_decode_init(VADriverContextP ctx,
     obj_surface->flags &= ~SURFACE_REF_DIS_MASK;
     obj_surface->flags |= (pic_param->pic_fields.bits.reference_pic_flag ? SURFACE_REFERENCED : 0);
     i965_check_alloc_surface_bo(ctx, obj_surface, 1, VA_FOURCC('N','V','1','2'), SUBSAMPLE_YUV420);
+
+    /* initial uv component for YUV400 case */
+    if (pic_param->seq_fields.bits.chroma_format_idc == 0) {
+         unsigned int uv_offset = obj_surface->width * obj_surface->height; 
+         unsigned int uv_size   = obj_surface->width * obj_surface->height / 2; 
+
+         drm_intel_gem_bo_map_gtt(obj_surface->bo);
+         memset(obj_surface->bo->virtual + uv_offset, 0x80, uv_size);
+         drm_intel_gem_bo_unmap_gtt(obj_surface->bo);
+    }
+
     gen6_mfd_init_avc_surface(ctx, pic_param, obj_surface);
 
     dri_bo_unreference(gen6_mfd_context->post_deblocking_output.bo);
@@ -978,7 +953,7 @@ gen6_mfd_avc_decode_init(VADriverContextP ctx,
     dri_bo_unreference(gen6_mfd_context->intra_row_store_scratch_buffer.bo);
     bo = dri_bo_alloc(i965->intel.bufmgr,
                       "intra row store",
-                      128 * 64,
+                      width_in_mbs * 64,
                       0x1000);
     assert(bo);
     gen6_mfd_context->intra_row_store_scratch_buffer.bo = bo;
@@ -987,7 +962,7 @@ gen6_mfd_avc_decode_init(VADriverContextP ctx,
     dri_bo_unreference(gen6_mfd_context->deblocking_filter_row_store_scratch_buffer.bo);
     bo = dri_bo_alloc(i965->intel.bufmgr,
                       "deblocking filter row store",
-                      30720, /* 4 * 120 * 64 */
+                      width_in_mbs * 64 * 4,
                       0x1000);
     assert(bo);
     gen6_mfd_context->deblocking_filter_row_store_scratch_buffer.bo = bo;
@@ -996,7 +971,7 @@ gen6_mfd_avc_decode_init(VADriverContextP ctx,
     dri_bo_unreference(gen6_mfd_context->bsd_mpc_row_store_scratch_buffer.bo);
     bo = dri_bo_alloc(i965->intel.bufmgr,
                       "bsd mpc row store",
-                      11520, /* 1.5 * 120 * 64 */
+                      width_in_mbs * 96,
                       0x1000);
     assert(bo);
     gen6_mfd_context->bsd_mpc_row_store_scratch_buffer.bo = bo;
@@ -1005,7 +980,7 @@ gen6_mfd_avc_decode_init(VADriverContextP ctx,
     dri_bo_unreference(gen6_mfd_context->mpr_row_store_scratch_buffer.bo);
     bo = dri_bo_alloc(i965->intel.bufmgr,
                       "mpr row store",
-                      7680, /* 1. 0 * 120 * 64 */
+                      width_in_mbs * 64,
                       0x1000);
     assert(bo);
     gen6_mfd_context->mpr_row_store_scratch_buffer.bo = bo;
@@ -1085,9 +1060,11 @@ gen6_mfd_mpeg2_decode_init(VADriverContextP ctx,
     struct i965_driver_data *i965 = i965_driver_data(ctx);
     struct object_surface *obj_surface;
     dri_bo *bo;
+    unsigned int width_in_mbs;
 
     assert(decode_state->pic_param && decode_state->pic_param->buffer);
     pic_param = (VAPictureParameterBufferMPEG2 *)decode_state->pic_param->buffer;
+    width_in_mbs = ALIGN(pic_param->horizontal_size, 16) / 16;
 
     mpeg2_set_reference_surfaces(
         ctx,
@@ -1109,7 +1086,7 @@ gen6_mfd_mpeg2_decode_init(VADriverContextP ctx,
     dri_bo_unreference(gen6_mfd_context->bsd_mpc_row_store_scratch_buffer.bo);
     bo = dri_bo_alloc(i965->intel.bufmgr,
                       "bsd mpc row store",
-                      11520, /* 1.5 * 120 * 64 */
+                      width_in_mbs * 96,
                       0x1000);
     assert(bo);
     gen6_mfd_context->bsd_mpc_row_store_scratch_buffer.bo = bo;
@@ -1378,6 +1355,7 @@ gen6_mfd_init_vc1_surface(VADriverContextP ctx,
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
     struct gen6_vc1_surface *gen6_vc1_surface = obj_surface->private_data;
+    int height_in_mbs = ALIGN(pic_param->coded_height, 16) / 16;
 
     obj_surface->free_private_data = gen6_mfd_free_vc1_surface;
 
@@ -1392,7 +1370,7 @@ gen6_mfd_init_vc1_surface(VADriverContextP ctx,
     if (gen6_vc1_surface->dmv == NULL) {
         gen6_vc1_surface->dmv = dri_bo_alloc(i965->intel.bufmgr,
                                              "direct mv w/r buffer",
-                                             557056,    /* 64 * 128 * 64 */
+                                             128 * height_in_mbs * 64,  /* scalable with frame height */
                                              0x1000);
     }
 }
@@ -1407,9 +1385,13 @@ gen6_mfd_vc1_decode_init(VADriverContextP ctx,
     struct object_surface *obj_surface;
     int i;
     dri_bo *bo;
+    int width_in_mbs;
+    int picture_type;
 
     assert(decode_state->pic_param && decode_state->pic_param->buffer);
     pic_param = (VAPictureParameterBufferVC1 *)decode_state->pic_param->buffer;
+    width_in_mbs = ALIGN(pic_param->coded_width, 16) / 16;
+    picture_type = pic_param->picture_fields.bits.picture_type;
 
     /* reference picture */
     obj_surface = SURFACE(pic_param->forward_reference_picture);
@@ -1449,7 +1431,7 @@ gen6_mfd_vc1_decode_init(VADriverContextP ctx,
     dri_bo_unreference(gen6_mfd_context->intra_row_store_scratch_buffer.bo);
     bo = dri_bo_alloc(i965->intel.bufmgr,
                       "intra row store",
-                      128 * 64,
+                      width_in_mbs * 64,
                       0x1000);
     assert(bo);
     gen6_mfd_context->intra_row_store_scratch_buffer.bo = bo;
@@ -1458,7 +1440,7 @@ gen6_mfd_vc1_decode_init(VADriverContextP ctx,
     dri_bo_unreference(gen6_mfd_context->deblocking_filter_row_store_scratch_buffer.bo);
     bo = dri_bo_alloc(i965->intel.bufmgr,
                       "deblocking filter row store",
-                      46080, /* 6 * 120 * 64 */
+                      width_in_mbs * 7 * 64,
                       0x1000);
     assert(bo);
     gen6_mfd_context->deblocking_filter_row_store_scratch_buffer.bo = bo;
@@ -1467,7 +1449,7 @@ gen6_mfd_vc1_decode_init(VADriverContextP ctx,
     dri_bo_unreference(gen6_mfd_context->bsd_mpc_row_store_scratch_buffer.bo);
     bo = dri_bo_alloc(i965->intel.bufmgr,
                       "bsd mpc row store",
-                      11520, /* 1.5 * 120 * 64 */
+                      width_in_mbs * 96,
                       0x1000);
     assert(bo);
     gen6_mfd_context->bsd_mpc_row_store_scratch_buffer.bo = bo;
@@ -1509,6 +1491,10 @@ gen6_mfd_vc1_decode_init(VADriverContextP ctx,
                 src_shift = !((src_h * width_in_mbs + src_w) & 1) * 4;
                 src_value = ((src[src_index] >> src_shift) & 0xf);
 
+                if (picture_type == GEN6_VC1_SKIPPED_PICTURE){
+                    src_value |= 0x2;
+                }
+
                 dst_index = src_w / 2;
                 dst[dst_index] = ((dst[dst_index] >> 4) | (src_value << 4));
             }
@@ -1634,9 +1620,19 @@ gen6_mfd_vc1_pic_state(VADriverContextP ctx,
 
     if (picture_type == GEN6_VC1_I_PICTURE || picture_type == GEN6_VC1_BI_PICTURE) /* I picture */
         trans_ac_y = pic_param->transform_fields.bits.transform_ac_codingset_idx2;
-    else
+    else {
         trans_ac_y = pic_param->transform_fields.bits.transform_ac_codingset_idx1;
-
+        /*
+         * 8.3.6.2.1 Transform Type Selection
+         * If variable-sized transform coding is not enabled,
+         * then the 8x8 transform shall be used for all blocks.
+         * it is also MFX_VC1_PIC_STATE requirement.
+         */
+        if (pic_param->transform_fields.bits.variable_sized_transform_flag == 0) {
+            pic_param->transform_fields.bits.mb_level_transform_type_flag   = 1;
+            pic_param->transform_fields.bits.frame_level_transform_type     = 0;
+        }
+    }
 
     if (picture_type == GEN6_VC1_B_PICTURE) {
         struct gen6_vc1_surface *gen6_vc1_surface = NULL;
@@ -1673,9 +1669,27 @@ gen6_mfd_vc1_pic_state(VADriverContextP ctx,
             brfd = 0;
     }
 
-    overlap = pic_param->sequence_fields.bits.overlap;
-    if (profile != GEN6_VC1_ADVANCED_PROFILE && pic_param->pic_quantizer_fields.bits.pic_quantizer_scale < 9)
-        overlap = 0;
+    overlap = 0;
+    if (profile != GEN6_VC1_ADVANCED_PROFILE){
+        if (pic_param->pic_quantizer_fields.bits.pic_quantizer_scale >= 9 &&
+            pic_param->picture_fields.bits.picture_type != GEN6_VC1_B_PICTURE) {
+            overlap = 1; 
+        }
+    }else {
+        if (pic_param->picture_fields.bits.picture_type == GEN6_VC1_P_PICTURE &&
+             pic_param->pic_quantizer_fields.bits.pic_quantizer_scale >= 9){
+              overlap = 1; 
+        }
+        if (pic_param->picture_fields.bits.picture_type == GEN6_VC1_I_PICTURE ||
+            pic_param->picture_fields.bits.picture_type == GEN6_VC1_BI_PICTURE){
+             if (pic_param->pic_quantizer_fields.bits.pic_quantizer_scale >= 9){
+                overlap = 1; 
+             } else if (va_to_gen6_vc1_condover[pic_param->conditional_overlap_flag] == 2 ||
+                        va_to_gen6_vc1_condover[pic_param->conditional_overlap_flag] == 3) {
+                 overlap = 1;
+             }
+        }
+    } 
 
     assert(pic_param->conditional_overlap_flag < 3);
     assert(pic_param->mv_fields.bits.mv_table < 4); /* FIXME: interlace mode */
@@ -2024,7 +2038,7 @@ gen6_dec_hw_context_init(VADriverContextP ctx, VAProfile profile)
 
     gen6_mfd_context->base.destroy = gen6_mfd_context_destroy;
     gen6_mfd_context->base.run = gen6_mfd_decode_picture;
-    gen6_mfd_context->base.batch = intel_batchbuffer_new(intel, I915_EXEC_RENDER);
+    gen6_mfd_context->base.batch = intel_batchbuffer_new(intel, I915_EXEC_RENDER, 0);
 
     for (i = 0; i < ARRAY_ELEMS(gen6_mfd_context->reference_surface); i++) {
         gen6_mfd_context->reference_surface[i].surface_id = VA_INVALID_ID;
