@@ -627,11 +627,9 @@ static VAStatus gen6_vme_constant_setup(VADriverContextP ctx,
                                         struct gen6_encoder_context *gen6_encoder_context)
 {
     struct gen6_vme_context *vme_context = &gen6_encoder_context->vme_context;
-    unsigned char *constant_buffer;
 
     dri_bo_map(vme_context->curbe.bo, 1);
     assert(vme_context->curbe.bo->virtual);
-    constant_buffer = vme_context->curbe.bo->virtual;
 	
     /*TODO copy buffer into CURB*/
 
@@ -672,19 +670,26 @@ static VAStatus gen6_vme_vme_state_setup(VADriverContextP ctx,
     return VA_STATUS_SUCCESS;
 }
 
-static void gen6_vme_pipeline_select(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+static void gen6_vme_pipeline_select(VADriverContextP ctx,
+                                     struct gen6_encoder_context *gen6_encoder_context,
+                                     struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
 
     BEGIN_BATCH(batch, 1);
     OUT_BATCH(batch, CMD_PIPELINE_SELECT | PIPELINE_SELECT_MEDIA);
     ADVANCE_BATCH(batch);
 }
 
-static void gen6_vme_state_base_address(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+static void gen6_vme_state_base_address(VADriverContextP ctx,
+                                        struct gen6_encoder_context *gen6_encoder_context,
+                                        struct intel_batchbuffer *batch)
 {
     struct gen6_vme_context *vme_context = &gen6_encoder_context->vme_context;
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
+
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
 
     BEGIN_BATCH(batch, 10);
 
@@ -709,11 +714,15 @@ static void gen6_vme_state_base_address(VADriverContextP ctx, struct gen6_encode
     ADVANCE_BATCH(batch);
 }
 
-static void gen6_vme_vfe_state(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+static void gen6_vme_vfe_state(VADriverContextP ctx,
+                               struct gen6_encoder_context *gen6_encoder_context,
+                               struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     struct gen6_vme_context *vme_context = &gen6_encoder_context->vme_context;
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     BEGIN_BATCH(batch, 8);
 
     OUT_BATCH(batch, CMD_MEDIA_VFE_STATE | 6);					/*Gen6 CMD_MEDIA_STATE_POINTERS = CMD_MEDIA_STATE */
@@ -732,11 +741,15 @@ static void gen6_vme_vfe_state(VADriverContextP ctx, struct gen6_encoder_context
 
 }
 
-static void gen6_vme_curbe_load(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+static void gen6_vme_curbe_load(VADriverContextP ctx,
+                                struct gen6_encoder_context *gen6_encoder_context,
+                                struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     struct gen6_vme_context *vme_context = &gen6_encoder_context->vme_context;
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     BEGIN_BATCH(batch, 4);
 
     OUT_BATCH(batch, CMD_MEDIA_CURBE_LOAD | 2);
@@ -748,11 +761,15 @@ static void gen6_vme_curbe_load(VADriverContextP ctx, struct gen6_encoder_contex
     ADVANCE_BATCH(batch);
 }
 
-static void gen6_vme_idrt(VADriverContextP ctx, struct gen6_encoder_context *gen6_encoder_context)
+static void gen6_vme_idrt(VADriverContextP ctx,
+                          struct gen6_encoder_context *gen6_encoder_context,
+                          struct intel_batchbuffer *batch)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     struct gen6_vme_context *vme_context = &gen6_encoder_context->vme_context;
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     BEGIN_BATCH(batch, 4);
 
     OUT_BATCH(batch, CMD_MEDIA_INTERFACE_LOAD | 2);	
@@ -767,14 +784,17 @@ static int gen6_vme_media_object(VADriverContextP ctx,
                                  struct encode_state *encode_state,
                                  int mb_x, int mb_y,
                                  int kernel,
-                                 struct gen6_encoder_context *gen6_encoder_context)
+                                 struct gen6_encoder_context *gen6_encoder_context,
+                                 struct intel_batchbuffer *batch)
 {
     struct i965_driver_data *i965 = i965_driver_data(ctx);
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
     struct object_surface *obj_surface = SURFACE(encode_state->current_render_target);
     int mb_width = ALIGN(obj_surface->orig_width, 16) / 16;
     int len_in_dowrds = 6 + 1;
 
+    if (batch == NULL)
+        batch = gen6_encoder_context->base.batch;
+
     BEGIN_BATCH(batch, len_in_dowrds);
     
     OUT_BATCH(batch, CMD_MEDIA_OBJECT | (len_in_dowrds - 2));
@@ -844,7 +864,8 @@ static void gen6_vme_pipeline_programing(VADriverContextP ctx,
                                          struct encode_state *encode_state,
                                          struct gen6_encoder_context *gen6_encoder_context)
 {
-    struct intel_batchbuffer *batch = gen6_encoder_context->base.batch;
+    struct i965_driver_data *i965 = i965_driver_data(ctx);
+    struct intel_batchbuffer *main_batch = gen6_encoder_context->base.batch;
     VAEncSliceParameterBuffer *pSliceParameter = (VAEncSliceParameterBuffer *)encode_state->slice_params[0]->buffer;
     VAEncSequenceParameterBufferH264 *pSequenceParameter = (VAEncSequenceParameterBufferH264 *)encode_state->seq_param->buffer;
     int is_intra = pSliceParameter->slice_flags.bits.is_intra;
@@ -852,8 +873,9 @@ static void gen6_vme_pipeline_programing(VADriverContextP ctx,
     int height_in_mbs = pSequenceParameter->picture_height_in_mbs;
     int emit_new_state = 1, object_len_in_bytes;
     int x, y;
+    struct intel_batchbuffer *batch = intel_batchbuffer_new(&i965->intel, I915_EXEC_RENDER, width_in_mbs * height_in_mbs * 8 * 4 + 0x200);
 
-    intel_batchbuffer_start_atomic(batch, 0x1000);
+    intel_batchbuffer_start_atomic(batch, width_in_mbs * height_in_mbs * 8 * 4 + 0x100);
 
     for(y = 0; y < height_in_mbs; y++){
         for(x = 0; x < width_in_mbs; x++){	
@@ -863,19 +885,19 @@ static void gen6_vme_pipeline_programing(VADriverContextP ctx,
                 intel_batchbuffer_emit_mi_flush(batch);
 
                 /*Step2: State command PIPELINE_SELECT*/
-                gen6_vme_pipeline_select(ctx, gen6_encoder_context);
+                gen6_vme_pipeline_select(ctx, gen6_encoder_context, batch);
 
                 /*Step3: State commands configuring pipeline states*/
-                gen6_vme_state_base_address(ctx, gen6_encoder_context);
-                gen6_vme_vfe_state(ctx, gen6_encoder_context);
-                gen6_vme_curbe_load(ctx, gen6_encoder_context);
-                gen6_vme_idrt(ctx, gen6_encoder_context);
+                gen6_vme_state_base_address(ctx, gen6_encoder_context, batch);
+                gen6_vme_vfe_state(ctx, gen6_encoder_context, batch);
+                gen6_vme_curbe_load(ctx, gen6_encoder_context, batch);
+                gen6_vme_idrt(ctx, gen6_encoder_context, batch);
 
                 emit_new_state = 0;
             }
 
             /*Step4: Primitive commands*/
-            object_len_in_bytes = gen6_vme_media_object(ctx, encode_state, x, y, is_intra ? VME_INTRA_SHADER : VME_INTER_SHADER, gen6_encoder_context);
+            object_len_in_bytes = gen6_vme_media_object(ctx, encode_state, x, y, is_intra ? VME_INTRA_SHADER : VME_INTER_SHADER, gen6_encoder_context, batch);
 
             if (intel_batchbuffer_check_free_space(batch, object_len_in_bytes) == 0) {
                 assert(0);
@@ -887,7 +909,29 @@ static void gen6_vme_pipeline_programing(VADriverContextP ctx,
         }
     }
 
-    intel_batchbuffer_end_atomic(batch);	
+    intel_batchbuffer_align(batch, 8);
+
+    BEGIN_BATCH(batch, 2);
+    OUT_BATCH(batch, 0);
+    OUT_BATCH(batch, MI_BATCH_BUFFER_END);
+    ADVANCE_BATCH(batch);
+
+    intel_batchbuffer_end_atomic(batch);
+
+    /* chain to the main batch buffer */
+    intel_batchbuffer_start_atomic(main_batch, 0x100);
+    intel_batchbuffer_emit_mi_flush(main_batch);
+    BEGIN_BATCH(main_batch, 2);
+    OUT_BATCH(main_batch, MI_BATCH_BUFFER_START | (2 << 6));
+    OUT_RELOC(main_batch,
+              batch->buffer,
+              I915_GEM_DOMAIN_COMMAND, 0,
+              0);
+    ADVANCE_BATCH(main_batch);
+    intel_batchbuffer_end_atomic(main_batch);
+
+    // end programing             
+    intel_batchbuffer_free(batch);
 }
 
 static VAStatus gen6_vme_prepare(VADriverContextP ctx, 
