@@ -35,13 +35,6 @@
 #include <intel_bufmgr.h>
 #include "i965_decoder.h"
 
-struct gen7_avc_surface
-{
-    dri_bo *dmv_top;
-    dri_bo *dmv_bottom;
-    int dmv_bottom_flag;
-};
-
 #define GEN7_VC1_I_PICTURE              0
 #define GEN7_VC1_P_PICTURE              1
 #define GEN7_VC1_B_PICTURE              2
@@ -53,6 +46,11 @@ struct gen7_avc_surface
 #define GEN7_VC1_ADVANCED_PROFILE       2
 #define GEN7_VC1_RESERVED_PROFILE       3
 
+#define GEN7_JPEG_ROTATION_0            0
+#define GEN7_JPEG_ROTATION_90           1
+#define GEN7_JPEG_ROTATION_270          2
+#define GEN7_JPEG_ROTATION_180          3
+
 #define GEN7_YUV400                     0
 #define GEN7_YUV420                     1
 #define GEN7_YUV422H_2Y                 2
@@ -87,6 +85,9 @@ struct gen7_mfd_context
     GenBuffer           bsd_mpc_row_store_scratch_buffer;
     GenBuffer           mpr_row_store_scratch_buffer;
     GenBuffer           bitplane_read_buffer;
+    
+    VASurfaceID jpeg_wa_surface_id;
+    dri_bo *jpeg_wa_slice_data_bo;
 
     int                 wa_mpeg2_slice_vertical_position;
 };
