@@ -41,12 +41,26 @@
 #define I965_MAX_ENTRYPOINTS                    5
 #define I965_MAX_CONFIG_ATTRIBUTES              10
 #define I965_MAX_IMAGE_FORMATS                  3
-#define I965_MAX_SUBPIC_FORMATS                 4
-#define I965_MAX_DISPLAY_ATTRIBUTES             4
+#define I965_MAX_SUBPIC_FORMATS                 6
+#define I965_MAX_SUBPIC_SUM                     4
 
 #define INTEL_STR_DRIVER_VENDOR                 "Intel"
 #define INTEL_STR_DRIVER_NAME                   "i965"
 
+#define I965_SURFACE_TYPE_IMAGE                   0
+#define I965_SURFACE_TYPE_SURFACE                 1
+
+#define I965_SURFACE_FLAG_FRAME                  0x00000000
+#define I965_SURFACE_FLAG_TOP_FIELD_FIRST        0x00000001
+#define I965_SURFACE_FLAG_BOTTOM_FIELD_FIRST     0x00000002
+
+struct i965_surface
+{
+    VAGenericID id;
+    int type;
+    int flags;
+};
+
 struct i965_kernel 
 {
     char *name;
@@ -81,6 +95,7 @@ struct decode_state
     struct buffer_store **slice_params;
     struct buffer_store *iq_matrix;
     struct buffer_store *bit_plane;
+    struct buffer_store *huffman_table;
     struct buffer_store **slice_datas;
     VASurfaceID current_render_target;
     int max_slice_params;
@@ -149,7 +164,9 @@ struct object_surface
 {
     struct object_base base;
     VASurfaceStatus status;
-    VASubpictureID subpic;
+    VASubpictureID subpic[I965_MAX_SUBPIC_SUM];
+    unsigned int subpic_render_idx;
+
     int width;
     int height;
     int size;
@@ -200,6 +217,7 @@ struct object_subpic
     int width;
     int height;
     int pitch;
+    float global_alpha;
     dri_bo *bo;
     unsigned int flags;
 };
@@ -208,6 +226,8 @@ struct hw_codec_info
 {
     struct hw_context *(*dec_hw_context_init)(VADriverContextP, VAProfile);
     struct hw_context *(*enc_hw_context_init)(VADriverContextP, VAProfile);
+    int max_width;
+    int max_height;
 };
 
 
@@ -229,6 +249,16 @@ struct i965_driver_data
     struct i965_render_state render_state;
     void *pp_context;
     char va_vendor[256];
+ 
+    VADisplayAttribute *display_attributes;
+    unsigned int num_display_attributes;
+    VADisplayAttribute *rotation_attrib;
+
+    /* VA/DRI (X11) specific data */
+    struct va_dri_output *dri_output;
+
+    /* VA/Wayland specific data */
+    struct va_wl_output *wl_output;
 };
 
 #define NEW_CONFIG_ID() object_heap_allocate(&i965->config_heap);
@@ -264,4 +294,11 @@ i965_check_alloc_surface_bo(VADriverContextP ctx,
                             unsigned int fourcc,
                             unsigned int subsampling);
 
+
+extern VAStatus i965_MapBuffer(VADriverContextP ctx,
+		VABufferID buf_id,       /* in */
+		void **pbuf);            /* out */
+
+extern VAStatus i965_UnmapBuffer(VADriverContextP ctx, VABufferID buf_id);
+
 #endif /* _I965_DRV_VIDEO_H_ */
